@@ -53,6 +53,8 @@ class GoogleCloudPilotAPI(abc.ABC):
     _scopes: List[str] = []
     _iam_roles: List[str] = []
     _cached_credentials: AuthType = None
+    _service_name = None
+    _google_managed_service = False  # Service agent requires impersonation
 
     def __init__(
             self,
@@ -166,21 +168,33 @@ class GoogleCloudPilotAPI(abc.ABC):
     def oidc_token(self) -> Dict[str, Dict[str, str]]:
         return {'oidc_token': {'service_account_email': self.credentials.service_account_email}}
 
-    async def add_permissions(self, email: str, project_id: str = None) -> None:
-        from gcp_pilot.resource import ResourceManager  # pylint: disable=import-outside-toplevel
+    async def set_up_permissions(self, email: str, project_id: str = None) -> None:
+        from gcp_pilot.resource import ResourceManager, ServiceAgent  # pylint: disable=import-outside-toplevel
 
+        rm = ResourceManager()
         for role in self._iam_roles:
-            await ResourceManager().add_member(
+            await rm.add_member(
                 email=email,
                 role=role,
                 project_id=project_id or self.project_id,
             )
 
+        if self._google_managed_service:
+            email = ServiceAgent.get_email(
+                service_name=f'{self._service_name} Service Account',
+                project_id=self.project_id,
+            )
+
+            await ResourceManager().allow_impersonation(
+                email=email,
+                project_id=project_id,
+            )
+
     def _get_project_number(self, project_id: str) -> int:
         from gcp_pilot.resource import ResourceManager  # pylint: disable=import-outside-toplevel
 
         project = ResourceManager().get_project(project_id=project_id)
-        return project.projectNumber
+        return project['projectNumber']
 
     def _as_duration(self, seconds):
         return Duration(seconds=seconds)
@@ -245,16 +259,25 @@ def friendly_http_error(func):
         'deleted': exceptions.AlreadyDeleted,
         'forbidden': exceptions.NotAllowed,
     }
+    _statuses = {
+        'INVALID_ARGUMENT': exceptions.ValidationError,
+    }
 
     def inner_function(*args, **kwargs):
         try:
             return func(*args, **kwargs)
         except HttpError as exc:
-            error = json.loads(exc.content)['error']['errors'][0]
+            errors = json.loads(exc.content)['error']
+
+            if 'errors' in errors:
+                exception_klass = _reasons.get(errors['errors'][0]['reason'], None)
+                details = ''  # TODO: add more details to the exceptions
+            else:
+                exception_klass = _statuses.get(errors['status'], None)
+                details = errors['message']
 
-            exception_klass = _reasons.get(error['reason'], None)
             if exception_klass:
-                raise exception_klass() from exc  # TODO: add more details to the exceptions
+                raise exception_klass(details) from exc
             raise exc
 
     return inner_function
